Astro = class("Astro")

--An entity on space
function Astro:initialize( radius )
	self.radius = radius
	self.x = 0
	self.y = 0
	self.children = {}
end

function Astro:addChild( child )
	table.insert(self.children,child)
end

function Astro:setOrbit( parent,radius,speed,radii )
	if parent and radius then
		self.orbit = {
			parent = parent,
			radius = radius,
			speed = speed or 0,
			pos = radii or 0,
		}
		parent:addChild(self)
	end
end

function Astro:getSpeed()
	if self.orbit then
		local speed = self.orbit.radius * self.orbit.speed
		local angle = self.orbit.pos + math.pi/2
		return speed,angle
	else
		return 0,0
	end
end

function Astro:getSpeedXY()
	if self.orbit then
		local speed, angle = self:getSpeed()
		local tx = self.x + speed * math.cos(angle)
		local ty = self.y + speed*math.sin(angle)
		return tx,ty
	else
		return self.x,self.y
	end
end

function Astro:update( dt )
	if self.orbit then
		self.orbit.pos = self.orbit.pos + (self.orbit.speed*dt)
		self.x = self.orbit.parent.x + self.orbit.radius*math.cos(self.orbit.pos)
		self.y = self.orbit.parent.y + self.orbit.radius*math.sin(self.orbit.pos)
	end
end

function Astro:isMouseOver()
	local mx,my = global.camera:mousePos()
	local mvec = math.sqrt(((mx+self.x)*(mx+self.x)) + ((my+self.y)*(my+self.y)))
	if mvec < self.radius then
			return true
	end
end

function Astro:mousepressed(x,y,button)
	if button == "r" then
		if self:isMouseOver(camera) then
			global.camera.target = self
		end
	end
end

function Astro:drawOrbit( ... )
	if self.orbit then
		if self.color then
			love.graphics.setColor(self.color.r,self.color.g,self.color.b,self.color.a or 255)
		end
		cx,cy,cr = global.camera:Displace(self.orbit.parent.x,self.orbit.parent.y,self.orbit.radius)
		love.graphics.circle("line",cx,cy,cr,40)
		love.graphics.white()
	end
end

function Astro:draw()
	if self.color then
		love.graphics.setColor(self.color.r,self.color.g,self.color.b,self.color.a or 255)
	end

	local dx,dy,dr = global.camera:Displace(self.x,self.y,self.radius)
	love.graphics.circle("fill",dx,dy,dr, 40)


	if self:isMouseOver() then
		love.graphics.setColor(255,255,255,255)
		love.graphics.setLineWidth(3)
		love.graphics.circle("line",dx,dy,dr, 40)
		love.graphics.setLineWidth(1)

		self:drawOrbit()

		for k,v in pairs(self.children) do
			v:drawOrbit()
		end		
	end
	love.graphics.white()
	--love.graphics.print("x:"..self.x.." y:"..self.y,dx,dy+dr+30)
end