Camera = class("Camera")

function Camera:initialize( target )
	self.x = 0
	self.y = 0
	self.target = target or nil
	self.zoom = 0.5
	self.min = 0.05
	self.max = 2
end

function Camera:screenshot()
	if not (love.filesystem.exists("screenshots")) then
		love.filesystem.mkdir("screenshots")
	end

	local screenshot = love.graphics.newScreenshot( )
	local date = os.date("*t")
	for k,v in pairs(date) do
		print(k,v)
	end
	screenshot:encode( "screenshots/"..date.year.."-"..date.month.."-"..date.day.."-"..date.hour.."-"..date.min.."-"..date.sec..".jpg" )
		
end

function Camera:update(  )
	if self.target and self.target.x and self.target.y then
		self:setPos(self.target.x*self.zoom,self.target.y*self.zoom)
	end

	if love.keyboard.isDown("e") then
		self.zoom = self.zoom + 0.0005
	end
	if love.keyboard.isDown("q") then
		self.zoom = self.zoom - 0.0005
	end
	if self.zoom < self.min then self.zoom = self.min end
	if self.zoom > self.max then self.zoom = self.max end
end

function Camera:setTarget( target )
	if target and target.x and target.y then
		self.target = target
	end
end

function Camera:Left()
	return (self.x or 0) - love.graphics.getWidth()/2 + 20
end

function Camera:Right()
	return (self.x or 0) + love.graphics.getWidth()/2 - 20
end

function Camera:Up()
	return (self.y or 0) - love.graphics.getHeight()/2 + 20
end

function Camera:Down()
	return (self.y or 0) + love.graphics.getHeight()/2 - 20
end

function Camera:Displace( x,y,r )
	local dx = x*self.zoom - self.x + love.graphics.getWidth()/2 
	local dy = -y*self.zoom + self.y + love.graphics.getHeight()/2
	local dr = (r or 0)*self.zoom
	return dx,dy,dr
end

function Camera:mousePos()
	local dx = ((love.mouse.getX() - (love.graphics.getWidth()/2)) + self.x)/self.zoom
	local dy = ((love.mouse.getY() - (love.graphics.getHeight()/2)) - self.y)/self.zoom
	return -dx,dy
end

function Camera:mouseVector( x,y )
	local x,y = self:Displace(x,y)
	local mx = love.mouse.getX()
	local my = love.mouse.getY()

	local dx = mx-x
	local dy = my-y

	local magnitude = math.sqrt( (dx*dx) + (dy*dy) )
	local angle = math.atan2(dy,dx)

	return magnitude,angle
end

function Camera:setPos(x,y)
	self.x = x
	self.y = y	
end

function Camera:draw()
end