System = class("System")

function System:initialize( name,x,y,objects )
	self.x = x
	self.y = y
	self.name = name
	self.background = {}
	self.content = {} or objects
	self.links = {}
	self.discovered = false
end

function System:hasLink( target )
	for k,v in pairs(self.links) do
		if v == target then
			return true
		end
	end
	return false
end

function System:addLink( target )
	if self:hasLink(target) then
		return "Link already existed"
	end
	table.insert(self.links,target)
	return "Link successfull"
end

function System:generate( links )
	self.background = {}
	params = {
		gravconstant = 0.03,
		sun = { 
			size = {
				min = 120,
				max = 500
			},
			mass = {
				min = 200,
				max = 500
			}
		},
		planets = {
			min = 1,
			max = 12,
			spacing = 300,
			solid = {
				mass = {
					min = 30,
					max = 90,
				},
				size = {
					min = 30,
					max = 90,
				},
				moon = {
					min = 0,
					max = 3
				}
			},
			gas = {
				mass = {
					min = 100,
					max = 200,
				},
				size = {
					min = 150,
					max = 300,
				},
				moon = {
					min = 1,
					max = 8
				}
			}
		},
		moons = {
			spacing = 30,
			mass = {
				min = 8,
				max = 40
			},
			size = {
				min=10,
				max=30
			}
		}
	}

	local sunSize = math.random(params.sun.size.min,params.sun.size.max)
	local sunMass = math.random(params.sun.mass.min,params.sun.mass.max)
	local sun = Sun(sunSize,sunMass)
	self:addBackground(sun)
	local dist = sun.radius * 2 + params.planets.spacing
	local planets = math.random(params.planets.min,params.planets.max)
	for i=1,planets do

		local color = {}
		if (math.random(1,5) > 2) then
			target = params.planets.solid
			color = {r=math.random(75,125),g=math.random(75,125),b=math.random(75,125)}
		else
			target = params.planets.gas
			color = {r=math.random(125,255),g=math.random(125,255),b=math.random(125,255)}
		end
		
		local size = math.random(target.size.min,target.size.max)
		local mass = math.random(target.mass.min,target.size.max)
		local planet = Planet(size,mass)
		planet.color = color
		local moons = math.random(params.planets.solid.moon.min,params.planets.solid.moon.max)
		local moondist = size
		for j=1,moons do
			local moonsize = math.random(params.moons.size.min,params.moons.size.max)
			local moon = Planet(moonsize)
			moon.color = {r=math.random(125,255),g=math.random(125,255),b=math.random(125,255)}
			moondist = moondist + moonsize + params.moons.spacing
			local radii = math.random(math.pi*2)
			local moonspeed = math.sqrt(mass*params.gravconstant/moondist)
			moon:setOrbit(planet,moondist,moonspeed,radii)
			moondist = moondist + moonsize
			self:addBackground(moon)
		end
		dist = dist + size+moondist + params.planets.spacing

		local speed = math.sqrt(sunMass*params.gravconstant/dist)
		local radii = math.random(math.pi*2)
		planet:setOrbit(sun,dist,speed,radii)

		dist = dist + size+moondist
		self:addBackground(planet)

	end

end

function System:getRandom()
	local num = math.random(1,#self.background)
	return self.background[num]
end

function System:getAt( x,y )
	local at = {}
	local all = {}
	for k,v in pairs(self.content) do
		table.insert(all,v)
	end
	for k,v in pairs(self.background) do
		table.insert(all,v)
	end
	for k,v in pairs(all) do
		if v.x and v.y then
			if v.radius then
				local mvec = math.sqrt(((x-v.x)*(x-v.x)) + ((y-v.y)*(y-v.y)))
				if mvec < v.radius then
					table.insert(at,v)
				end
			end
		end
	end
	return at
end

function System:update( dt )
	if global.sys == self then
		self.discovered = true
	end
	for k,v in pairs(self.background) do
		if v.update then
			v:update(dt)
		end
	end
	for k,v in pairs(self.content) do
		if v.update then
			v:update(dt)
		end
	end
end

function System:mousepressed(x,y,button,camera)
	for k,v in pairs(self.background) do
		if v.mousepressed then
			v:mousepressed(x,y,button,camera)
		end
	end
	for k,v in pairs(self.content) do
		if v.mousepressed then
			v:mousepressed(x,y,button,camera)
		end
	end
end

function System:addBackground( obj )
	table.insert(self.background,obj)
end
function System:add(obj)
	table.insert(self.content,obj)
end

function System:draw(camera)
	for k,v in pairs(self.background) do
		if v.draw then
			v:draw(camera)
		end
	end
	for k,v in pairs(self.content) do
		if v.draw then
			v:draw(camera)
		end
	end
	love.graphics.print("System: "..self.name,30,60)
	love.graphics.print("WASD to move, Q/E to zoom in/out, R to regenerate current system, F to jump at red jumpgates",30,30)
end

System.listOfNames = 
{
	"Aquarius",
	"Andromeda",
	"Aquila",
	"Aries",
	"Auriga",
	"Bootes",
	"Cancer",
	"Canes Venatici",
	"Canis Major",
	"Capricornus",
	"Carina",
	"Cassiopeia",
	"Centaurus",
	"Cepheusº",
	"Cetus",
	"Columba",
	"Coma Berenices",
	"Corona Australis",
	"Corona Borealis",
	"Corvus",
	"Crater",
	"Crux",
	"Cygnus",
	"Delphinus",
	"Draco",
	"Equuleus",
	"Eridanus",
	"Gemini",
	"Grus",
	"Hercules",
	"Hydra",
	"Leo",
	"Leo Minor",
	"Lepus",
	"Libra",
	"Lynx",
	"Lyra",
	"Octans",
	"Ophiuchus",
	"Orion",
	"Pavo",
	"Pegasus",
	"Perseus",
	"Phoenix",
	"Pisces",
	"Piscis Austrinus",
	"Puppis",
	"Sagitta",
	"Sagittarius",
	"Scorpius",
	"Serpens",
	"Taurus",
	"Triangulum",
	"Triangulum Australe",
	"Ursa Major",
	"Ursa Minor",
	"Vela",
	"Virgo",
	"Vulpecula"
}